// $Id: linux_threads.h,v 1.5 2002/10/16 16:59:47 mourlouk Exp $
// Copyright (C) 2001 Wi_LAN Technologies, Inc
//
// $Log: linux_threads.h,v $
// Revision 1.5  2002/10/16 16:59:47  mourlouk
// Improved way of retrieving MAC addresses - VAL support
//
// Revision 1.3  2001/11/12 21:55:34  mourlouk
// Included errno.h at linux_threads.c
//
// Revision 1.2  2001/10/30 01:07:45  lpoulsen
// Add "module ID" statement, audit trail generator and copyright line
// Remove "win_or_linux.h" and use a -D in Linux makefile instead.
//
//
#define __USE_GNU  
#include <pthread.h>     // pthread functions and data structures 

int send_thr_id, read_thr_id, receive_thr_id, file_thr_id;
pthread_t  send_thread;
pthread_t  read_thread;
pthread_t  receive_thread;
pthread_t  file_thread;


