// $Id: cio.h,v 1.8 2008/08/21 18:23:41 lpoulsen Exp $
// Copyright (C) 2001 Wi_LAN Technologies, Inc
//
// $Log: cio.h,v $
// Revision 1.8  2008/08/21 18:23:41  lpoulsen
// Works with scripts on v4 embedded protocols
//
// Revision 1.7  2008/07/11 17:10:27  lpoulsen
// Resolve compiler warnings.
//
// Revision 1.6  2008/07/11 16:28:46  lpoulsen
// Econ 4.00 release candidate supports UPDATE and SELECT commands.
//
// Revision 1.5  2008/05/31 00:35:09  lpoulsen
// Econ 4.x preview - now with command shell
//
// Revision 1.4  2008/05/10 01:17:37  lpoulsen
// Making econ -x filename work (not yet complete)
//
// Revision 1.3  2006/04/27 19:27:14  lpoulsen
// Allow for pathnames longer than 24 bytes.
//
// Revision 1.2  2001/10/30 01:07:45  lpoulsen
// Add "module ID" statement, audit trail generator and copyright line
// Remove "win_or_linux.h" and use a -D in Linux makefile instead.
//
//
#define ECON_KEYBOARD 11
#define ECON_SOCKET 22
#define ECON_FILE 33

#define ESC_KEY 8027
#define F1_KEY 8080
#define F2_KEY 8081
#define F3_KEY 8082
#define F4_KEY 8083
#define F5_KEY 8084
#define F6_KEY 8085
#define F7_KEY 8086
#define UNKNOWN_KEY 8099

#define isNormalChar(ch) (ch<8080?1:0)

FILE *console_out_file;

extern int debug_level;

extern int read_from_keyboard(unsigned char *data, int max_data_size);
extern int read_from_socket(unsigned char *data, int max_data_size);
extern int read_from_file(unsigned char *data, int max_data_size);
extern char ctrl(char control_key);
extern void e_print(char c);
extern boolean fileExist(char *filename);
extern uint32 filelen(char *filename);
extern void reverseColorSettings();
extern void initCIOSettings();
extern void restoreCIOSettings();
