// $Id: netcardr.h,v 1.2 2001/10/30 01:07:45 lpoulsen Exp $
// Copyright (C) 2001 Wi_LAN Technologies, Inc
//
// $Log: netcardr.h,v $
// Revision 1.2  2001/10/30 01:07:45  lpoulsen
// Add "module ID" statement, audit trail generator and copyright line
// Remove "win_or_linux.h" and use a -D in Linux makefile instead.
//
//
#ifndef _NETCARDR_H_
#define _NETCARDR_H_

// Copyright And Configuration Management ----------------------------------
//
//                                 Header For
//          Windis NetCard Registry Enumeration Module - NetCardR.h
//                                    For
//                         Windows NT And Windows 2000
//
//        Copyright (c) 2000 Printing Communications Associates, Inc.
//                                - PCAUSA -
//                         <http://www.pcausa.com>
//
//                             Thomas F. Divine
//                           4201 Brunswick Court
//                        Smyrna, Georgia 30080 USA
//                              (770) 432-4580
//
//                       <mailto:tdivine@pcausa.com>
//
// End ---------------------------------------------------------------------


BOOLEAN MAII_IsWindowsNT( void );
BOOLEAN MAII_IsWindows2000( void );
BOOLEAN MAII_IsWindows95( void );

typedef
struct _MAII_ADAPTER_INFO
{
   TCHAR   cDescription[ _MAX_PATH ];
   TCHAR   cTitle[ _MAX_PATH ];        // e.g., "[1] Realtek RTL 8029 PCI Adapter"

   TCHAR   cServiceName[ _MAX_PATH ];  // NT Service Name: e.g., RTL80291
                                       // 9X Enumerator  : e.g., "0001"
}
   MAII_ADAPTER_INFO, *PMAII_ADAPTER_INFO;

DWORD MAII_GetFirstAdapterRegistryInfo(
   PMAII_ADAPTER_INFO pAdapterInfo
   );

DWORD MAII_GetNextAdapterRegistryInfo(
   PMAII_ADAPTER_INFO pAdapterInfo
   );

#endif // _NETCARDR_H_

