// This is to satisfy references to these functions in libnet libraries.
//
// These functions are never called
//
// $Log: gethost_dummy.c,v $
// Revision 1.4  2006/06/06 02:03:29  lpoulsen
// First attempt at compiling on MAC OS X
//
// Revision 1.3  2006/04/28 02:36:48  lpoulsen
// Resolve merge error.
//
// Revision 1.2  2006/04/28 02:33:28  lpoulsen
// Add more routines. Linker has trouble deciding if getaddrinfo() is
//    available for static linking or not.
//
//
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#ifdef __APPLE__
#define __socklen_t socklen_t
#endif

struct hostent *
gethostbyname(const char *name)
{
    printf("dummy gethostbyname() called with arg=%s\n", name);
    return NULL;
}

struct hostent *
gethostbyaddr(const void *addr, __socklen_t len, int type)
{
    printf("dummy gethostbyaddr(-,%d, %d) called\n", len, type);
    return NULL;
}

struct hostent *
gethostbyname2(const char *name, int af)
{
    printf("dummy gethostbyname(%s, %d) called\n", name, af);
    return NULL;
}
struct netent *
getnetbyname(const char *name)
{
    printf("dummy getnetbyname() called with arg=%s\n", name);
    return NULL;
}
#if 0
int
getaddrinfo(const char *node, const char *service,
                              const struct addrinfo *hints,
                              struct addrinfo **res)
{
    printf("dummy getaddrinfo(%s, %s,,) called\n", node, service);
    return -1;
}
#endif
struct protoent *
getprotobyname(const char *name)
{
    printf("dummy getprotobyname(%s) called\n", name);
    return NULL;
}
struct servent *
getservbyname(const char *name, const char *proto)
{
    printf("dummy getservbyname(%s, %s) called\n", name, proto);
    return NULL;
}
