// $Id: windows_threads.h,v 1.5 2002/10/16 16:59:47 mourlouk Exp $
// Copyright (C) 2001 Wi_LAN Technologies, Inc
//
// $Log: windows_threads.h,v $
// Revision 1.5  2002/10/16 16:59:47  mourlouk
// Improved way of retrieving MAC addresses - VAL support
//
// Revision 1.3  2002/05/14 23:44:01  mourlouk
// more minor changes ...
//
// Revision 1.2  2001/10/30 01:07:45  lpoulsen
// Add "module ID" statement, audit trail generator and copyright line
// Remove "win_or_linux.h" and use a -D in Linux makefile instead.
//
//
#include <windows.h>
#include <process.h>
#include <conio.h>
#include <windowsx.h>

HANDLE SendThreadHandle;
HANDLE ReadThreadHandle;
HANDLE ReceiveThreadHandle;
HANDLE FileThreadHandle;

HANDLE ThreadTerminationEvent;
HANDLE ActivateWatchDog;
HANDLE WatchDogThreadHandle;
HANDLE BabySitterThreadHandle;
