// $Id: win_emul.h,v 1.6 2011/08/29 22:29:27 lpoulsen Exp $
// Copyright (C) 2001 Wi_LAN Technologies, Inc
//
// $Log: win_emul.h,v $
// Revision 1.6  2011/08/29 22:29:27  lpoulsen
// Clean compilation on FC3 linux.
//
// Revision 1.5  2006/04/28 02:13:41  lpoulsen
// Last file for the "longer path" fix
//
// Revision 1.4  2002/05/21 17:53:50  lpoulsen
// Fix many compiler warnings.
//
// Revision 1.3  2001/11/13 00:38:51  mourlouk
// ....................
//
// Revision 1.2  2001/10/30 01:07:45  lpoulsen
// Add "module ID" statement, audit trail generator and copyright line
// Remove "win_or_linux.h" and use a -D in Linux makefile instead.
//
//

#define __USE_GNU

#include <unistd.h>
#include <pthread.h>     // pthread functions and data structures 

extern int EnterCriticalSection(pthread_mutex_t* my_mutex);
extern int LeaveCriticalSection(pthread_mutex_t* my_mutex);
extern int ResetEvent(pthread_cond_t* my_condition);
extern int SetEvent(pthread_cond_t* my_condition);
extern int WaitForSingleObject(pthread_cond_t* my_condition,int timeout);

#define INFINITE      0xFFFF
#define WAIT_OBJECT_0 0
#define WAIT_TIMEOUT  0x102
#define WAIT_FAILED   0xFFFFFFFF

#define Sleep(A) sleep(A/1000)

#ifndef _MAX_PATH
#define _MAX_PATH 80
#endif
