// $Id: bld_date.c,v 1.1 2011/03/22 00:26:10 lpoulsen Exp $
// Copyright (c) 2011 Afar Communications Inc, Santa Barbara, CA 93117 USA
//
// $Log: bld_date.c,v $
// Revision 1.1  2011/03/22 00:26:10  lpoulsen
// Resolve sequence errors on very weak links
// Resolve hangs when Ctrl-C between "dir" and "download"
// Resolve conflicting build date information
//

char cvs_tag[] = "$Name: ECON_04-25 $";
char bld_date[] = __DATE__;
char bld_time[] = __TIME__;

// void bld_date_dummy(void) { return; }

// End
