// $Id: ioctlnqgs.h,v 1.2 2001/10/30 01:07:45 lpoulsen Exp $
// Copyright (C) 2001 Wi_LAN Technologies, Inc
//
// $Log: ioctlnqgs.h,v $
// Revision 1.2  2001/10/30 01:07:45  lpoulsen
// Add "module ID" statement, audit trail generator and copyright line
// Remove "win_or_linux.h" and use a -D in Linux makefile instead.
//
//
#ifndef _IOCTLNQGS_H_
#define _IOCTLNQGS_H_

// Copyright And Configuration Management ----------------------------------
//
//                                 Header For
//       IOCTL NDIS Query Global Statistics (NQGS) Module - IoctlNQGS.h
//                                    For
//                         Windows NT And Windows 2000
//
//        Copyright (c) 2000 Printing Communications Associates, Inc.
//                                - PCAUSA -
//
//                             Thomas F. Divine
//                           4201 Brunswick Court
//                        Smyrna, Georgia 30080 USA
//                              (770) 432-4580
//                            tdivine@pcausa.com
// 
// End ---------------------------------------------------------------------

DWORD
MAII_IoctlNdisQueryGlobalStats(
   LPCTSTR  lpAdapterName,
   ULONG    OidCode,
   PVOID    InformationBuffer,
   UINT     InformationBufferLength,
   PUINT    pBytesWritten
   );

#endif // _IOCTLNQGS_H_

