//** Debug flag bits
extern int      debug_level;
extern uint32   debug_flags;

#define DBFLG_DOWNLOAD     0x0010
#define DBFLG_ACK_SEQ      0x0020
#define DBFLG_SESSION_PCAP 0x0040
#define DBFLG_SESSION_TEXT 0x0080
#define DBFLG_KEEP_TRACES  0x0100
#define DBFLG_DATE_TIME    0x0200
#define DBFLG_THREADS      0x0400
#define DBFLG_ADAPTER      0x0800
#define DBFLG_NETHEALTH    0x1000
#define DBFLG_ECON_PKTS    0x2000

