// $Id: afartype.h,v 1.5 2014/05/21 00:09:36 lpoulsen Exp $
//
// Copyright (c) 2008 by Afar Communications Inc, Santa Barbara, CA 93117 USA
//
// Afar's standard data types
//
// $Log: afartype.h,v $
// Revision 1.5  2014/05/21 00:09:36  lpoulsen
// Redefine int32 and uint32 so they work on 64-bit compilers
//   (ECON still won't compile in 64-bit mode because you cannot
//   cast a pointer into a uint32. And we don't want to build it
//   in 64-bit mode anyway, because we would have to maintain two
//   versions: 32-bit and 64-bit.)
//
// Revision 1.4  2011/02/04 22:29:06  lpoulsen
// Many little fixes.
//
// Revision 1.2  2009/08/06 17:02:33  lpoulsen
// NRS Toolkit 1.15
//
// Revision 1.2  2008/08/29 23:44:11  lpoulsen
// Allow printing of ATC packet headers in print-udp-packets
//
// Revision 1.1  2008/08/21 21:58:02  lpoulsen
// PCAP file dump utility
//
// Revision 1.2  2008/06/10 00:06:10  lpoulsen
// New -X option is now working
// Updated -I and -O implementation
//
// Revision 1.1  2008/05/31 00:35:09  lpoulsen
// Econ 4.x preview - now with command shell
//
//
#ifndef _AFARTYPE_H
#define _AFARTYPE_H  // load this file only once

#ifndef __cplusplus
typedef enum { false=0, true } BOOL;
#endif

typedef unsigned char  byte;
typedef unsigned char  BYTE;

typedef unsigned char  boolean;
typedef          char  int8;
typedef unsigned char  uint8;
typedef unsigned char  uchar;
typedef short int      int16;
typedef unsigned short uint16;
typedef          int   int32;
typedef unsigned int   uint32;
typedef uint32         DWORD;
typedef long long int          int64;
typedef unsigned long long int uint64;
typedef uint64                 time64_t;        // a FILETIME in disguise

typedef uint8          macAddr[6];
typedef uint8          ipAddr[4];
typedef uint8          ip_addr[4];

typedef uint32         ucio_time_t;

#define                AOK       0

// Utility functions in afarutil.c
#ifndef _INC_TIME
#include <time.h>
#endif

extern char *afar_date(const time_t t);
extern char *afar_time(const time_t t);
extern int   strncmp_nocase(const char *s1, const char *s2, const size_t n);

#endif
// End
